<?php
// +---------------------------------------------------------------------------+
// | GUS Plugin                                                                |
// +---------------------------------------------------------------------------+
// | functions.inc                                                             |
// | This file does two things: 1) it implements the necessary Geeklog Plugin  |
// | API method and 2) implements all the common code needed by the stats      |
// | PHP files.                                                                |
// |                                                                           |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2002, 2003, 2005 by the following authors:                  |
// |                                                                           |
// | Authors: Andy Maloney      - asmaloney@users.sf.net                       |
// |          Tom Willett       - twillett@users.sourceforge.net               |
// |          John Hughes       - jlhughes@users.sf.net                        |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

$langfile = $_CONF['path'] . 'plugins/gus/language/' . $_CONF['language'] . '.php';

if ( file_exists( $langfile ) ) 
    include_once( $langfile );
else 
    require_once( $_CONF['path'] . 'plugins/gus/language/english.php' );

require_once( $_CONF['path'] . 'plugins/gus/config.php' );

///////////////////////////////////////////////////////////////////
///  Set up config variables and call to add stats to database  //
/////////////////////////////////////////////////////////////////

GUS();

function GUS()
{
	global $_GUS_VARS, $_USER, $_TABLES;
	
	
	$_GUS_VARS = array();
	
	$rec = DB_query( "SELECT * FROM {$_TABLES['gus_vars']}", 1 );
	
	if ( DB_numRows( $rec ) > 0 )
	{
		// Get the GUS vars from the DB
		while ( $row = mysql_fetch_assoc( $rec ) )
		{
			$_GUS_VARS[$row['name']] = $row['value'];
		}
	
		// store the MySQL version number in $_GUS_VARS['sql_version']
		$version = mysql_get_server_info();
		
		$_GUS_VARS['sql_version'] = array();
		
		if ( eregi( "^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})", $version, $match ) )
		{
			$_GUS_VARS['sql_version']['major'] = $match[1];
			$_GUS_VARS['sql_version']['minor'] = $match[2];
			$_GUS_VARS['sql_version']['patch'] = $match[3];
		}
		
		if ( $_GUS_VARS['capture'] == '1' )
		{
			// First check the IP...
			$_GUS_VARS['remote_ip'] = GetIP( $_SERVER['REMOTE_ADDR'] );

			$rec = DB_query( "SELECT COUNT(*) AS ignored
								FROM {$_TABLES['gus_ignore_ip']}
								WHERE ip = SUBSTRING( '{$_GUS_VARS['remote_ip']}', 1, LENGTH( ip ) )
								LIMIT 1", 1 );
								
			$row = DB_fetchArray( $rec, false );
	
			if ( $row['ignored'] == '1' )
				return;
	
			// Next check the page...
			$this_page = substr( $_SERVER['SCRIPT_NAME'], 1 );	// substr to get everything after the leading '/'
			
			$rec = DB_query( "SELECT COUNT(*) AS ignored FROM {$_TABLES['gus_ignore_page']} WHERE page = '{$this_page}' LIMIT 1", 1 );
			$row = DB_fetchArray( $rec, false );
			
			if ( $row['ignored'] )
				return;
	
			// Finally check the user...
			if ( $_USER['uid'] < 2 )
			{
				$user_id = 1; 
				$user_name = 'Anonymous';
			}
			else
			{
				$user_id = $_USER['uid'];
				$user_name = $_USER['username'];
			}
			
			$rec = DB_query( "SELECT COUNT(*) AS ignored FROM {$_TABLES['gus_ignore_user']} WHERE username = '{$user_name}' LIMIT 1", 1 );
			$row = DB_fetchArray( $rec, false );
			
			if ( $row['ignored'] )
				return;
		
			// Finally, add stats
			GUS_addStats( $user_id, $user_name );
		}
	}
}

// adapted from code found here: http://www.php.net/getenv
function GetIP( $ip )
{
// commented out because it seems to give misleading results sometimes
/*
	if ( $_SERVER["HTTP_CLIENT_IP"] && strcasecmp($_SERVER["HTTP_CLIENT_IP"], "unknown") )
		$ip = $_SERVER["HTTP_CLIENT_IP"];
	else if ( $_SERVER["HTTP_X_FORWARDED_FOR"] && strcasecmp($_SERVER["HTTP_X_FORWARDED_FOR"], "unknown") )
		$ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
*/	
	// IF more than one IP, grab the first
	if ( strstr( $ip, ', ' ) )
	{
		$ips = explode(', ', $ip);
		$ip = $ips[0];
	}
	
	return( $ip );
}

// adapted from code found here: http://www.php.net/gethostbyaddr
function ip_to_hostname( $ip )
{
	global $_GUS_CONF;
	
	$timeout = $_GUS_CONF['host_lookup_timeout'];
	
	if ( $_GUS_CONF['use_nslookup'] )
	{
		$res = `nslookup -timeout=$timeout $ip`;
		
		if ( preg_match( '/\nName:(.*)\n/', $res, $out ) )
			return trim( $out[1] );
	}
	else
	{
		$output = `host -W $timeout $ip`;
		
		if ( ereg( '.*pointer ([A-Za-z0-9.-]+)\..*', $output, $regs ) )
			return $regs[1];
	}
	
	return $ip;
}

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementations                                        |
// +---------------------------------------------------------------------------+

/**
 * Returns the current plugin code version
 * 
 * @return string    plugin version
*/
function plugin_chkVersion_gus()
{
	return '1.3.2';
}

// Not currently supported by GL, but hopefully in 1.3.12
function	plugin_enableStateChange_gus( $enable )
{
	// enable/disable the Who's online block    
    return true;
}

// Add the CSS file if we are browsing a GUS page
function	plugin_getheadercode_gus()
{
    global $_CONF;
        
    $this_dir = basename( dirname( $_SERVER['SCRIPT_NAME'] ) );

	if ( $this_dir != 'gus' )
		return '';
		
    // check for CSS file specific to this theme
 	$css_file = 'css/' . $_CONF['theme'] . '.css';
 	
 	// IF we don't find one that's readable THEN use the default 	
    if ( !is_readable( $css_file ) )
	{
	   	$css_file = 'css/default.css';
   
		// IF we still don't have anything THEN return
		if ( !is_readable( $css_file ) )
			return '';
	}
	
  	$stylesheet = $_CONF['site_url'] . '/gus/' . $css_file;

	return '<link rel="stylesheet" type="text/css" href="' . $stylesheet . '" title="' . $_CONF['theme'] . '">';
}

/**
* Returns the items for this plugin that should appear on the main menu
*
* NOTE: this MUST return the url/value pairs in the following format
* $<arrayname>[<label>] = <url>
*
*/
function plugin_getmenuitems_gus()
{
    global $_CONF, $LANG_GUS00, $_GUS_anon_access, $_GUS_enable_main_menu_GUS, $_GUS_enable_main_menu_privacy_policy;
    
    $menuitems = array();
    
    if ( $_GUS_enable_main_menu_GUS && ($_GUS_anon_access || SEC_hasRights( 'gus.view' )) )
    {
        $tmp = $LANG_GUS00['main_menu_title'];
        $menuitems["$tmp"] = $_CONF['site_url'] . '/gus/index.php';
    }
    
    if ( $_GUS_enable_main_menu_privacy_policy == 1 )
    {
        $tmp = $LANG_GUS00['priv_pol'];
        $menuitems["$tmp"] = $_CONF['site_url'] . '/gus/privpol.php';
    }
    
    return $menuitems;
}

/**
* Shows the statistics for the stats plugin on stats.php.  If
* $showsitestats is 1 then we are to only print the overall stats in the 'site
* statistics' box otherwise we show the detailed stats for the stats
*
* @showsitestate        int         Flag to let us know which stats to get
*/
function plugin_showstats_gus( $showsitestats ) 
{
	global $_TABLES, $LANG_GUS_builtin_stats, $_GUS_stats;
	
    if ( ($showsitestats == 1) && ($_GUS_stats == 1) )
	{
        $rec = DB_query( "SELECT COUNT( DISTINCT ip ) AS visitors FROM {$_TABLES['gus_userstats']}" );
		$row = DB_fetchArray( $rec, false );
        $retval = "<table border = '0' width='100%' cellspacing='0' cellpadding='0'>";
        $retval .= "<tr><td>" . $LANG_GUS_builtin_stats['unique_visitors'] . "</td>";
        $retval .= "<td align='right'>" . $row['visitors'] . "&nbsp;&nbsp </td></tr>";
		
        $total_stats = DB_count( $_TABLES['users'] );
        $retval .= "<tr><td>" . $LANG_GUS_builtin_stats['users'] . "</td>";
        $retval .= "<td align='right'>" . $total_stats . "&nbsp;&nbsp </td></tr></table>";
    }
	else
	{
	    $retval = '';
	}
	
	return $retval;
}

/**
* This will put an option for stats in the command and control block on
* moderation.php
*
*/
function plugin_cclabel_gus() 
{
	global $LANG_GUS00, $_CONF, $_GUS_IMG_name;
	
	
	$retval = array();
	
	if ( SEC_hasRights('gus.admin') )
	{
		$retval = array( $LANG_GUS00['GUS_title'],
			$_CONF['site_admin_url'] . '/plugins/gus/index.php',
			$_CONF['site_url'] . '/gus/images/' . $_GUS_IMG_name );
	}
	
	return $retval;
}

/**
* returns the administrative option for this plugin
*
*/
function plugin_getadminoption_gus() 
{
    global $LANG_GUS_blocks, $_CONF;
    

    if ( SEC_hasRights( 'gus.admin' ) )
    {
        return array( $LANG_GUS_blocks['admin_menu_title'], $_CONF['site_admin_url'] . '/plugins/gus/index.php', 0 );
    }
}

/**
* returns the user option for this plugin
*
*/
function plugin_getuseroption_gus() 
{
    global $LANG_GUS_blocks, $_CONF, $_GUS_user;


    if ( SEC_hasRights( 'gus.view' ) && ($_GUS_user == 1) )
    {
    	$title = $LANG_GUS_blocks['user_menu_title'];    	

    	// Micro-hack:
    	//	We're only supposed to return one thing, but we close the anchor and open another
    	//	in order to return two...
    	$title .= '</a> <a href="' . $_CONF['site_url'] . '/gus/day.php' . '">[' . $LANG_GUS_blocks['today'] . ']';

        return array( $title, $_CONF['site_url'] . '/gus/index.php', 0 );
    }
}

// upgrade to this version of GUS
function plugin_upgrade_gus()
{
	global $_TABLES;
	
	$res = DB_query( "SELECT pi_version FROM {$_TABLES['plugins']} WHERE pi_name = 'gus'" );
	$row = DB_fetchArray( $res, false );
	
	COM_errorLog( "Updating GUS plugin from v" . $row['pi_version'] . ' to v' . plugin_chkVersion_gus(), 1 );
	
	return GUS_doUpgrades( $row['pi_version'] );
}

function GUS_doUpgrades( $current_gus_version )
{
	global $_TABLES, $_CONF, $_DB;

	$_DB->setDisplayError( true );


	$done = false;

	while ( $done == false )
	{
		switch ( $current_gus_version )
		{
			case '1.0.0':
			case '1.1.0':
			case '1.2.0':
				$current_gus_version = '1.3.0';
				break;
				
			case '1.3.0':
				require_once( $_CONF['path'] . 'plugins/gus/sql/updates/gus_1.3.0_to_1.3.1.php' );

				for ( $i = 1; $i <= count( $_SQL ); $i++ )
				{
					DB_query( current( $_SQL ) );
					next( $_SQL );
				}

				$current_gus_version = '1.3.1';
				break;
			
			case '1.3.1':
				$current_gus_version = '1.3.2';
				break;
				
			default:
				$done = true;
				break;
		}
	}

	DB_query( "UPDATE {$_TABLES['plugins']} SET pi_version = '{$current_gus_version}' WHERE pi_name = 'gus'" );
	
	return true;
}

/**
* Removes the datastructures for this plugin from the Geeklog database
*/  
function plugin_uninstall_gus()
{
    global $_TABLES;

    $FEATURES = array ( 'gus.admin', 'gus.view');
    $TABLES = array ( 'gus_userstats', 'gus_user_agents', 'gus_ignore_ip', 'gus_ignore_user', 'gus_ignore_page', 'gus_vars' );

    COM_errorLog( 'Removing the GUS plugin...', 1 );
    
    // Unregister the plugin with Geeklog
    GUS_log_and_execute( "DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'gus' LIMIT 1" );

    // Drop tables we created
    COM_errorLog( "Removing tables", 1 );
    foreach ( $TABLES as $table )
    {
        $t = $_TABLES[$table];
        
        GUS_log_and_execute( "DROP TABLE IF EXISTS {$t}" );
    }

    // remove block
    COM_errorLog( "Removing gus block", 1 );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['blocks']} WHERE name='gus_block' LIMIT 1" );

    // Remove Security for this plugin
    $grp_id = DB_getItem( $_TABLES['vars'], 'value', "name = 'gus_group_id'" );
    
    COM_errorLog( "Removing gus admin group", 1 );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['groups']} WHERE grp_id = $grp_id", 1 );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['vars']} WHERE name = 'gus_group_id'" );
    
    COM_errorLog( "Removing root users from admin of gus" );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['group_assignments']} WHERE ug_main_grp_id = $grp_id", 1 );
    COM_errorLog( "Removing group id in vars" );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['vars']} WHERE value = $grp_id", 1 );
    
    // Remove all the associated features -- access rights
    foreach ( $FEATURES as $feature )
    {
		COM_errorLog( "Removing feature '{$feature}' and rights", 1 );
        $feat_id = DB_getItem($_TABLES['features'], 'ft_id', "ft_name = '$feature'");
        GUS_log_and_execute( "DELETE FROM {$_TABLES['access']} WHERE acc_ft_id = $feat_id", 1 );
        GUS_log_and_execute( "DELETE FROM {$_TABLES['features']} WHERE ft_id = $feat_id", 1 );
    }

    COM_errorLog( "Removing the {$_TABLES['vars']} reference to gus" );
    GUS_log_and_execute( "DELETE FROM {$_TABLES['vars']} WHERE value = 'gus'", 1 );

    COM_errorLog( 'The GUS plugin has been removed', 1 );
    
    return true;
}

//////////////////////////////////////////////////////////////////////////////////
//  STATS Functions                                                             //
//////////////////////////////////////////////////////////////////////////////////

// Check for the 'stats' plugin, which may be installed under a different name
//	return true/false
function GUS_checkStatsInstall()
{
	global	$_CONF, $_TABLES, $_DB_table_prefix, $_ST_plugin_name;
	
	if ( empty( $_ST_plugin_name ) )
	{
		$_ST_plugin_name = DB_getItem( $_TABLES['vars'], 'value', "name = 'stats_name'" );
	
		if ( empty( $_ST_plugin_name ) )
		   return false;

		// include the config file so we get the $_TABLE vars
		require_once( $_CONF['path'] . 'plugins/' . $_ST_plugin_name . '/config.php' );
		
		// now that we have the table names, check that the old userstats table actually exists
		$res = DB_query( "SELECT uid FROM {$_TABLES['userstats']} LIMIT 1", 1 );
		
		return( !DB_error() );
	}
	
	return true;
}

// Because the referrers are stored encoded, we need our site URL to look similar for SQL comparison
//	- find the 'http://foo.bar.com' part
//	- make it look like the referrers by encoding it
//	- escape '%' for the LIKE clause
function GUS_getEncodedSiteURL()
{
	global $_CONF;
	
	preg_match( "/^(http:\/\/[^\/]+)/i", $_CONF['site_url'], $matches );
	$encoded_site_url = urlencode( addslashes( $matches[1] ) );
	$encoded_site_url = str_replace( '%', '\%', $encoded_site_url );
	
	return $encoded_site_url;
}

/*
* Return an array with Browser type and Version
*/
function GUS_getBrowser( $UA )
{    
    $browser = array();
    
    if ( @eregi( "opera[ /]([0-9]{1,2}.[0-9]{1,3}){0,1}", $UA, $match ) )
    {
        $browser['type'] = "Opera"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "konqueror/([\.\+[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = "Konqueror"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "lynx/([\.\+[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = "Lynx"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "links \(([0-9]{1,2}.[0-9]{1,3})", $UA, $match ) )
    {
        $browser['type'] = "Links"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "(Safari|Camino|Firefox|Firebird)/([\.\+[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = $match[1]; 
        $browser['ver'] = $match[2];
    }
    elseif ( eregi( "msie ([\.\+[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = "MSIE"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "^Mozilla/5.0 \(compatible; Yahoo\! Slurp;", $UA ) )
    {
        $browser['type'] = "Yahoo! Slurp"; 
        $browser['ver'] = '1.0';        
    }
    elseif ( eregi( "netscape6/(6.[0-9]{1,3})", $UA, $match ) )
    {
        $browser['type'] = "Netscape"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "netscape/(7.[0-9]{1,3})", $UA, $match ) )
    {
        $browser['type'] = "Netscape"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "^mozilla/5.+ rv:([^\)]+)\)", $UA, $match ) )
    {
        $browser['type'] = "Mozilla"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "^mozilla/([\.\+[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = "Netscape"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "^Jigsaw/([^ ]+)", $UA, $match ) )
    {
        $browser['type'] = "W3C CSS Validator"; 
        $browser['ver'] = $match[1];
    }
    elseif ( eregi( "w3m", $UA ) )
    {
        $browser['type'] = "w3m"; 
        $browser['ver'] = "Unknown";
    }
    elseif ( eregi( "^([^/]+)/([\.[:digit:]]+)", $UA, $match ) )
    {
        $browser['type'] = strtr( $match[1], '_', ' ' ); 
        $browser['ver'] = $match[2];
    }
    else
    {
        $browser['type'] = "Unknown"; 
        $browser['ver'] = "Unknown";
    }
    
    return $browser;
}

/*
* Return Computer Type
*/
function GUS_getComputerType( $UA ) 
{
	if ( eregi( "linux", $UA ) )
	{
    	$return = "Linux";
	}
	elseif ( eregi( "win32", $UA ) )
	{
    	$return = "Windows";
	}
	elseif ( ( eregi( "(win)([0-9]{2})", $UA, $match ) ) || ( eregi( "(windows) ([0-9]{2})", $UA, $match ) ) )
	{
    	$return = "Windows $match[2]";
	}
	elseif ( eregi( "win ?9x", $UA ) )
	{
    	$return = "Windows 9x";
	}
	elseif ( eregi( "Windows XP", $UA ) )
	{
    	$return = "Windows XP";
	}
	elseif ( eregi( "(winnt)([0-9]{1,2}.[0-9]{1,2}){0,1}", $UA, $match ) )
	{
		if ($match[2] == '5.1') $return = "Windows XP";
		elseif ($match[2] == '5.0') $return = "Windows 2000";
		elseif ($match[2] == '5.2') $return = "Windows 2003";
		elseif ($match[2] == '6.0') $return = "Windows Vista";
		else $return = "Windows NT $match[2]";
	}
	elseif ( eregi( "(windows nt) {0,1}([0-9]{1,2}.[0-9]{1,2}){0,1}", $UA, $match ) )
	{
		if ($match[2] == '5.1') $return = "Windows XP";
		elseif ($match[2] == '5.0') $return = "Windows 2000";
		elseif ($match[2] == '5.2') $return = "Windows 2003";
		elseif ($match[2] == '6.0') $return = "Windows Vista";
		else $return = "Windows NT $match[2]";
	}
	elseif ( eregi( "mac os x", $UA ) )
	{
    	$return = "Mac OS X";
	}
	elseif ( eregi( "mac", $UA ) )
	{
    	$return = "MacOS";
	}
	elseif ( eregi( "darwin", $UA ) )
	{
    	$return = "Darwin";
	}
	elseif ( eregi( "(sunos) ([0-9]{1,2}.[0-9]{1,2}){0,1}", $UA, $match ) )
	{
    	$return = "SunOS $match[2]";
 	}
 	elseif ( eregi( "(beos) r([0-9]{1,2}.[0-9]{1,2}){0,1}", $UA, $match ) )
 	{
    	$return = "BeOS $match[2]";
    }
    elseif ( eregi( "freebsd", $UA ) )
    {
        $return = "FreeBSD";
    }
    elseif ( eregi( "openbsd", $UA ) )
    {
        $return = "OpenBSD";
    }
    elseif ( eregi( "irix", $UA ) )
    {
        $return = "IRIX";
    }
    elseif ( eregi( "os/2", $UA ) )
    {
        $return = "OS/2";
    }
    elseif ( eregi( "plan9", $UA ) )
    {
        $return = "Plan9";
    }
    elseif ( eregi( "unix", $UA ) || eregi( "hp-ux", $UA ) )
    {
        $return = "Unix";
    }
    elseif ( eregi( "osf", $UA ) )
    {
        $return = "OSF";
    }
 	elseif ( eregi( "Symbian", $UA ) )
 	{
    	$return = "SymbianOS";
    }
 	elseif ( eregi( "WebTV", $UA ) )
 	{
    	$return = "WebTV";
    }
    elseif ( eregi( "(bot|Slurp|larbin|Jeeves|Spider)", $UA ) )
    {
        $return = "[Robot]";
    }
    else
    {
        $return = "Unknown";
    }
          
    return $return;
}
                        

/*
* add stats to database
*/
function GUS_addStats( $user_id, $user_name )
{
    global $_TABLES, $_GUS_VARS, $QUERY_STRING;

	$user_agent = addslashes( substr( $_SERVER['HTTP_USER_AGENT'], 0, 128 ) );
	
	$sql = "SELECT ua_id FROM {$_TABLES['gus_user_agents']} WHERE user_agent = '{$user_agent}' LIMIT 1";
	$res = DB_query( $sql );
	
	if ( $row = DB_fetchArray( $res ) )
	{
		$ua_id = $row[0];
	}
	else
	{
		$browser = GUS_getBrowser( $_SERVER['HTTP_USER_AGENT'] );
		$platform = GUS_getComputerType( $_SERVER['HTTP_USER_AGENT'] );

		$sql = "INSERT INTO {$_TABLES['gus_user_agents']} SET
			user_agent = '{$user_agent}',
			browser = '" . $browser['type'] . "',
			version = '" . $browser['ver'] . "',
			platform = '" . $platform . "'";

		$res = DB_query( $sql );
	
		$ua_id = DB_insertId();
	}
	
	// SPECIAL CASE: 404s fill in the query string witht the URI the user was trying to access
	if ( preg_match( "/^.+404\.php$/", $_SERVER['SCRIPT_NAME'] ) )
	{
		$query_string = $_SERVER['REQUEST_URI'];
	}
	else
	{
		$query_string = $_SERVER['QUERY_STRING'];
		
		// This fixes a problem reported by Tom whereby $_SERVER['QUERY_STRING'] and $QUERY_STRING are not
		//	the same [as they should be...]
		if ( $query_string == '' )
			$query_string = $QUERY_STRING;
	}
	
	$sql = "INSERT INTO " . $_TABLES['gus_userstats'] . " SET"
		. " uid='" . $user_id
		. "', username='" . $user_name
		. "', ip='" . $_GUS_VARS['remote_ip']
		. "', host='" . ip_to_hostname( $_GUS_VARS['remote_ip'] )
		. "', date='" . date( "Y-m-d" )
		. "', time='" . date( "G:i:s" )
		. "', referer='" . urlencode( addslashes( $_SERVER['HTTP_REFERER'] ) )
		. "', query_string='" . urlencode( addslashes( $query_string ) )
		. "', request='" . $_SERVER['REQUEST_METHOD']
		. "', page='" . substr( $_SERVER['SCRIPT_NAME'], 1 )
		. "', ua_id='" . $ua_id . "'";
		
   DB_query( $sql );
}

/**
* Caching Functions
*/

/* 
* Function to return cache file name for current page
*/
function GUS_cachefile()
{
    global $_CONF;

    $cfile = basename( $_SERVER['SCRIPT_NAME'] ) . '?' . $_SERVER['QUERY_STRING'];
    $cfile = str_replace('?','_',$cfile);
    $cfile = str_replace('&','_',$cfile);
    $cfile = str_replace('.','_',$cfile);
    $cfile = str_replace('=','_',$cfile);
    $cfile = $_CONF['path_html'] . 'gus/cache/' . $cfile . '.html';
    return $cfile;
}

/*
* Function to return current cache file in string variable
*/

function GUS_getcache()
{
    $buffer = '';
    if ($fd = fopen(GUS_cachefile(),"r")) {
        while (!feof($fd)) {
            $buffer .= fgets($fd,1024);
        }
        fclose($fd);
    } else {
        COM_errorlog( "GUS Cache File " . GUS_cachefile() . " not found", 1 );
    }
    return $buffer;
}

/*
* Function to write string to cache file
*/
function GUS_writecache($buf)
{
    if ($fd = fopen(GUS_cachefile(),"w")) {
        fputs($fd,$buf);
        fclose($fd);
    } else {
        COM_errorlog( "Unable to open GUS Cache file " . GUS_cachefile() . " for writing", 1 );
    }
}


function GUS_log_and_execute( $sql )
{
    COM_errorLog( $sql, 1 );
    DB_query( $sql, 1 );
}

/*
* Stats block to show whos online now, # visitors today, registered today, new today, referers today
*
* @return   string  formatted html for block
*/
function phpblock_gusstats()
{
    global $_USER, $_CONF, $_TABLES, $LANG01, $LANG_GUS_wo, $_GUS_CONF;
	
    // Code borrowed from lib-common.php
    
    if( empty( $_USER['uid'] ) OR $_USER['uid'] == 1 )
    {
        // The following code handles anonymous users so they show up properly
        DB_query( "DELETE FROM {$_TABLES['sessions']} WHERE remote_ip = '{$_SERVER['REMOTE_ADDR']}' AND uid = 1" );
        // Build a useless sess_id (needed for insert to work properly)
        mt_srand(( double )microtime() * 1000000 );
        $sess_id = mt_rand();
        $curtime = time();
        // Insert anonymous user session
        DB_query( "INSERT INTO {$_TABLES['sessions']} (sess_id, start_time, remote_ip, uid)
        			VALUES ($sess_id,$curtime,'{$_SERVER['REMOTE_ADDR']}',1)" );
    }

	$expire_time = time() - $_CONF['whosonline_threshold'];
   
    // Clear out any expired sessions
    DB_query( "DELETE FROM {$_TABLES['sessions']} WHERE uid = 1 AND start_time < " . $expire_time );

    // Code adapted from whose online block

    if ( $_GUS_CONF['wo_fullname'] )
        $byname = 'fullname, username';
    else
        $byname = 'username';

    $num_anon = 0;
    $num_reg  = 0;
	$retval = '';
	
    if ( $_GUS_CONF['wo_online'] )
	{        
        $result = DB_query( "SELECT DISTINCT {$_TABLES['sessions']}.uid, {$byname}, photo, showonline
        		FROM {$_TABLES['sessions']},{$_TABLES['users']},{$_TABLES['userprefs']}
        		WHERE {$_TABLES['sessions']}.uid <> 1
        			AND {$_TABLES['users']}.uid = {$_TABLES['sessions']}.uid
        			AND {$_TABLES['users']}.uid = {$_TABLES['userprefs']}.uid
        			AND start_time >= $expire_time 
        		ORDER BY {$byname}" );

		while ( $row = DB_fetchArray( $result, false ) )
        {
            if ( $row['showonline'] == 1 )
            {
                if ( $_GUS_CONF['wo_fullname'] && !empty( $row['fullname'] ) )
                    $username = $row['fullname'];
                else
                    $username = $row['username'];

				$retval .= '<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $row['uid'] . '">'
							. $username . '</a>';
                
                if ( ($_CONF['allow_user_photo'] == 1) && !empty( $row['photo'] ) ) 
                {
                    $retval .= '&nbsp;<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $row['uid'] . '">';
                    $retval .= '<img src="' . $_CONF['site_url'] . '/gus/images/user_photo.gif" width=16 height=12 border="0" alt=""></a>';
                }
                
                $retval .= '<br>';
				$num_reg++;
            }
            else
            {
				// this user does not want to show up in Who's Online
                $num_anon++;
            }
        }

		if ( $_GUS_CONF['wo_users_anonymous'] && ( empty( $_USER['uid'] ) || ( $_USER['uid'] == 1 ) ) )
		{
			// note that we're overwriting the contents of $retval here
			if( $num_reg > 0 )
				$retval = $LANG01[112] . ': ' . $num_reg . '<br>';
			else
				$retval = '';
		}
        
        $result = DB_query( "SELECT COUNT( DISTINCT uid, remote_ip ) AS nonames FROM {$_TABLES['sessions']} WHERE uid = 1" );
    }
	else
	{
        $result = DB_query( "SELECT COUNT( DISTINCT uid, remote_ip ) AS nonames FROM {$_TABLES['sessions']}" );
    }
	
	$row = DB_fetchArray( $result, false );
    $num_anon += $row['nonames'];
    
	if ( $num_anon > 0 )
    {
        if ( $_GUS_CONF['wo_online'] ) 
            $retval .= $LANG01[41];
		else
            $retval .= $LANG01[17];

        $retval .= ': ' . $num_anon . '<br>';
    }
	
    if ( $_GUS_CONF['wo_daily'] )
	{
        $retval .= "<table width='100%'><tr><td width='20%'><hr></td>
        			<td nowrap>{$LANG_GUS_wo['stats']}</td>
       				<td width='60%'><hr></td></tr></table>";
		
        $result = DB_query( "SELECT COUNT(*) AS pages FROM {$_TABLES['gus_userstats']} WHERE date = CURRENT_DATE()" );
        $row = DB_fetchArray( $result, false );
        $retval .= $row['pages'] . ' ' . $LANG_GUS_wo['page_title'] . '<br>';
		
        $result = DB_query( "SELECT COUNT( DISTINCT ip ) AS visitors FROM {$_TABLES['gus_userstats']} WHERE date = CURRENT_DATE()" );
        $row = DB_fetchArray( $result, false );
        $retval .= $row['visitors'] . ' ' . $LANG_GUS_wo['unique_visitors'] . '<br>';
    }
	
    if ( $_GUS_CONF['wo_registered'] )
	{
		$result = DB_query( "SELECT DISTINCT uid FROM {$_TABLES['gus_userstats']} WHERE uid > 1 AND date = CURRENT_DATE()" );

		$num_reg = DB_numRows( $result );

		if ( $num_reg > 0 )
		{
			$retval .= "<table width='100%'><tr><td width='20%'><hr></td>
						<td nowrap>{$LANG_GUS_wo['reg_users']}</td>
						<td width='60%'><hr></td></tr></table>";

			if ( $_GUS_CONF['wo_users_anonymous'] && ( empty( $_USER['uid'] ) || ( $_USER['uid'] == 1 ) ) )
			{
				$retval .= $LANG01[112] . ': ' . $num_reg . '<br>';
			}
			else
			{			
				while ( $row = DB_fetchArray( $result, false ) )
				{					
					$rec = DB_query( "SELECT uid, username, fullname, photo
								FROM {$_TABLES['users']}
								WHERE uid = '" . $row['uid'] . "' LIMIT 1" );
			
					$A = DB_fetchArray( $rec, false );
					
					$retval .= '<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $A['uid'] . '">';
					
					if ( $_GUS_CONF['wo_fullname'] && !empty( $A['fullname'] ) )
						$retval .= $A['fullname'];
					else
						$retval .= $A['username'];
					
					$retval .= '</a>';
					
					if ( ($_CONF['allow_user_photo'] == 1) && !empty( $A['photo'] ) ) 
					{
						$retval .= '&nbsp;<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $A['uid'] . '">';
						$retval .= '<img src="' . $_CONF['site_url'] . '/gus/images/user_photo.gif" width=16 height=12 border="0" alt=""></a>';
					}
					
					$retval .= '<br>';
				}
			}
		}
    }
	
    if ( $_GUS_CONF['wo_new'] )
	{
		$result = DB_query( "SELECT uid, username, fullname, photo FROM {$_TABLES['users']}
								WHERE regdate >= DATE_SUB( CURRENT_DATE(), INTERVAL 1 DAY )" );

		$num_new = DB_numRows( $result );

		if ( $num_new > 0 )
		{
			$retval .= "<table width='100%'><tr><td width='20%'><hr></td>
						<td nowrap>{$LANG_GUS_wo['new_users']}</td>
						<td width='60%'><hr></td></tr></table>";
		
			if ( $_GUS_CONF['wo_users_anonymous'] && ( empty( $_USER['uid'] ) || ( $_USER['uid'] == 1 ) ) )
			{
				$retval .= $LANG_GUS_wo['new_users'] . ': ' . $num_new . '<br>';
			}
			else
			{			
				while ( $row = DB_fetchArray( $result, false ) )
				{			
					$retval .= '<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $row['uid'] . '">';
					
					if ( $_GUS_CONF['wo_fullname'] && !empty( $row['fullname'] ) )
						$retval .= $row['fullname'];
					else
						$retval .= $row['username'];
					
					$retval .= '</a>';
					
					if ( ($_CONF['allow_user_photo'] == 1) && !empty( $row['photo'] ) )
					{
						$retval .= '&nbsp;<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&amp;uid=' . $row['uid'] . '">';
						$retval .= '<img src="' . $_CONF['site_url'] . '/gus/images/user_photo.gif" width=16 height=12 border="0" alt=""></a>';
					}
					
					$retval .= '<br>';
				}
			}
		}
    }
	
    if ( $_GUS_CONF['wo_refs'] )
	{
		$encoded_site_url = GUS_getEncodedSiteURL();
		
		$result = DB_query( "SELECT referer FROM {$_TABLES['gus_userstats']}
							WHERE referer <> ''
								AND date = CURRENT_DATE()
								AND referer NOT LIKE '{$encoded_site_url}%'" );

		$header_output = false;
		
		while ( $row = DB_fetchArray( $result, false ) )
		{			
			if ( !$header_output )
			{
				$retval .= "<table width='100%'><tr><td width='20%'><hr></td>
					<td nowrap>{$LANG_GUS_wo['referers']}</td>
					<td width='60%'><hr></td></tr></table>";
				
				$header_output = true;
			}
			
			$referrer = urldecode( $row['referer'] );
            $url = parse_url( $referrer );
          
            if ( $url['host'] != '' )
            {
				$host = preg_replace( '/^www\./', '', $url['host'], 1 );
				
				if ( $host != '' )
				{
					$ref["$host"] += 1;
					$refurl["$host"] = htmlentities( preg_replace( '/admin\/plugins.*$/', '', $referrer, 1 ) );
				}
            }
        }
		
        if ( is_array($ref) )
		{
            arsort( $ref );
            $i = 0;
            
            foreach ( $ref as $key => $value )
			{
                $retval .= '<a href="' . $refurl["$key"] . '">' . $key . '</a> - ' . $value . '<br>';
				
				$i++;
                
                if ( $i >= $_GUS_CONF['wo_max_referrers'] )
                	break;
            }
        }
    }
	
    return $retval;
}

/*==================================================\
| This does the URLS for page navigation and header |
\==================================================*/

function GUS_create_url( $ignore )
{
	global $_CONF, $SCRIPT_NAME, $_GET;
	
	$firstvar = true;
	
	foreach($_GET as $key => $var)
	{
		if ( ($key != $ignore) && ($key != 'page') ) // always ignore page 
		{
			if ($firstvar) // first time through
			{
				$qstring .= "?" . $key . "=" . $var;
				$firstvar = false;
			}
			else
			{
				$qstring .= "&" . $key . "=" . $var;
			}
		}
	}
	
	$newurl = parse_url( $_CONF['site_url'] );
	
	$retval = $newurl['scheme'] . "://" . $newurl['host'] . $SCRIPT_NAME . $qstring;
	
	return htmlentities( $retval );
}
?>
