<?php
// +---------------------------------------------------------------------------+
// | GUS Plugin                                                                |
// +---------------------------------------------------------------------------+
// | sql.inc                                                                   |
// |                                                                           |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2005 by the following authors:                              |
// |                                                                           |
// | Authors: Andy Maloney      - asmaloney@users.sf.net                       |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+


function GUS_create_temp_userstats_table( $year, $month )
{
	global	$_GUS_VARS, $_TABLES, $_GUS_table_prefix;
	
	$tmp_name = $_GUS_table_prefix . 'temp_' . $year . $month;

	// CREATE TABLE ... LIKE only exists on MySQL >= 4.1
	//	See: http://dev.mysql.com/doc/mysql/en/create-table.html
	if ( $_GUS_VARS['sql_version']['major'] >= 4 && $_GUS_VARS['sql_version']['minor'] >= 1 )
	{
		DB_query( "CREATE TEMPORARY TABLE $tmp_name LIKE {$_TABLES['gus_userstats']}", 1 );
	}
	else
	{
		GUS_create_temp_table( $tmp_name, "TEMPORARY" );
	}
	
	// IF there was a problem creating the TEMPORARY TABLE
	//	THEN use a regular table
	if ( DB_error() )
	{		
		DB_query( "DROP TABLE IF EXISTS $tmp_name" );
		
		GUS_create_temp_table( $tmp_name );
	}
	
	$Ym_date_format = "$year-" . ($month < 10 ? '0' : '' ) . $month;
	
	DB_query( "INSERT INTO $tmp_name SELECT * FROM {$_TABLES['gus_userstats']}
		WHERE date LIKE '{$Ym_date_format}%'" );
	
	return $tmp_name;
}

function GUS_create_temp_table( $table_name, $temp = '' )
{
	DB_query( "CREATE $temp TABLE $table_name (
				  uid int(10) NOT NULL default '1',
				  ip varchar(20) default '0.0.0.0',
				  host varchar(75) default '',
				  page varchar(50) default '',
				  username varchar(16) default '',
				  referer varchar(255) default '',
				  request varchar(10) default '',
				  query_string varchar(255) default '',
				  `date` date default NULL,
				  `time` time NOT NULL default '00:00:00',
				  ua_id smallint(5) unsigned NOT NULL default '0',
				  KEY uid (uid),
				  KEY ip (ip),
				  KEY page (page),
				  KEY refer (referer),
				  KEY ua_id (ua_id),
				  KEY `date` (`date`)
				  )", 1 );
}

function GUS_get_date_comparison( $field, $year, $month, $day = '' )
{
	global	$_GUS_VARS;
  
  
	$date_formatted = "$year-" . ($month < 10 ? '0' : '' ) . $month;
	
	if ( $day )
		$date_formatted .= '-' . ($day < 10 ? '0' : '' ) . $day;

	return "$field LIKE '{$date_formatted}%'";
}

function GUS_get_order_by( $sort )
{
	$sort_str = '';
	
	// sort options
	switch ( $sort ) 
	{
		case 'ipASC':
		   $sort_str .= "ORDER BY ip ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'ipDESC':
		   $sort_str .= "ORDER BY ip DESC, date DESC, time DESC, username ASC";
		   break;
		   
		case 'viewsASC':
		   $sort_str .= "ORDER BY views ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'viewsDESC':
		   $sort_str .= "ORDER BY views DESC, date DESC, time DESC, username ASC";
		   break;

		case 'typeASC':
		   $sort_str .= "ORDER BY request ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'typeDESC':
		   $sort_str .= "ORDER BY request DESC, date DESC, time DESC, username ASC";
		   break;
		   
		case 'ptuASC':
		   $sort_str .= "ORDER BY page ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'ptuDESC':
		   $sort_str .= "ORDER BY page DESC, date DESC, time DESC, username ASC";
		   break;
		   
		case 'usernameASC':
		   $sort_str .= "ORDER BY username ASC, date ASC, time ASC";
		   break;
		   
		case 'usernameDESC':
		   $sort_str .= "ORDER BY username DESC, date DESC, time DESC";
		   break;
		   
		case 'hostASC':
		   $sort_str .= "ORDER BY host ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'hostDESC':
		   $sort_str .= "ORDER BY host DESC, date DESC, time DESC, username ASC";
		   break;
		   
		case 'refererASC':
		   $sort_str .= "ORDER BY referer ASC, date ASC, time ASC, username ASC";
		   break;
		   
		case 'refererDESC':
		   $sort_str .= "ORDER BY referer DESC, date DESC, time DESC, username ASC";
		   break;
		   
		case 'dateASC':
		   $sort_str .= "ORDER BY date ASC, time ASC, username ASC";
		   break;
		   
		default: // same as date DESC
		   $sort_str .= "ORDER BY date DESC, time DESC, username ASC";
		   break;
	}
	
	return $sort_str;
}

?>
